@@ -31,7 +31,7 @@ benchmarkGeneratorNKL <- function(N=10,K=1,PI=1:K,g=NULL){
 	if(is.null(g)){ # generate the fitness subfunctions
 		g <- matrix(runif(N*2^(K+1)),N)
 	}
-	bits = 2^(0:K)
+	bits <- 2^(0:K)
 	bits
 	g
 	N
@@ -41,7 +41,7 @@ benchmarkGeneratorNKL <- function(N=10,K=1,PI=1:K,g=NULL){
 		usum=0
 		for(i in 1:N){
 			xx <- x[c(i,((i+PI-1)%%N)+1)] #select current and impacting (neighbouring) bits (circular)
-			usum=usum+ g[i,sum(bits*xx)+1]
+			usum <- usum+ g[i,sum(bits*xx)+1]
 		} 
 		-usum/N #minus for minimization
 	}
