@@ -1,13 +1,12 @@
-#   Copyright (c) 2014-2015 by Martin Zaefferer, Cologne University of Applied Sciences
-
 ###################################################################################
 #' Unimodal Fitness Landscape
 #' 
 #' This function generates uni-modal fitness landscapes based on distance measures.
-#' The fitness is the distance to a reference individual. Hence, the reference individual
-#' is the optimum of the landscape. 
+#' The fitness is the distance to a reference individual or center. Hence, the reference individual
+#' is the optimum of the landscape. This function is essentially a wrapper
+#' for the \code{\link{landscapeGeneratorMUL}}
 #'
-#' @param ref Reference individual
+#' @param ref reference individual
 #' @param distanceFunction Distance function, used to evaluate d(x,ref), where x is an arbitrary new individual
 #'
 #' @return returns a function. The function requires a list of candidate solutions as its input, where each solution is suitable for use with the distance function. The function returns a numeric vector.
@@ -31,12 +30,8 @@
 #'
 #' @export
 ###################################################################################
-landscapeGeneratorUNI <- function(ref=1:10,distanceFunction){  #N= number of elements of a string, K= number of neighbours, PI = relative position of each neighbour in string, g= set of fitness functions for each combination of string components
-	distanceFunction #lazy evaluation fix, faster than force()
-	ref #lazy evaluation fix, faster than force()
-	function(x){
-		distanceVector(ref,x,distanceFunction)
-	}
+landscapeGeneratorUNI <- function(ref,distanceFunction){  #N= number of elements of a string, K= number of neighbours, PI = relative position of each neighbour in string, g= set of fitness functions for each combination of string components
+	landscapeGeneratorMUL(list(ref),distanceFunction)
 }
 
 
@@ -44,10 +39,10 @@ landscapeGeneratorUNI <- function(ref=1:10,distanceFunction){  #N= number of ele
 #' Multimodal Fitness Landscape
 #' 
 #' This function generates multi-modal fitness landscapes based on distance measures.
-#' The fitness is the minimal distance to several reference individuals. Hence, each reference individual
+#' The fitness is the minimal distance to several reference individuals or centers. Hence, each reference individual
 #' is an optimum of the landscape. 
 #'
-#' @param ref list of reference individuals
+#' @param ref list of reference individuals / centers
 #' @param distanceFunction Distance function, used to evaluate d(x,ref[[n]]), where x is an arbitrary new individual
 #'
 #' @return returns a function. The function requires a list of candidate solutions as its input, where each solution is suitable for use with the distance function. The function returns a numeric vector.
@@ -70,10 +65,11 @@ landscapeGeneratorUNI <- function(ref=1:10,distanceFunction){  #N= number of ele
 #'
 #' @export
 ###################################################################################
-landscapeGeneratorMUL <- function(ref=list(1:10),distanceFunction){
+landscapeGeneratorMUL <- function(ref,distanceFunction){
 	distanceFunction #lazy evaluation fix, faster than force()
 	ref #lazy evaluation fix, faster than force()
 	function(x){
+    if(!is.list(x))x<-list(x)
 		k = length(ref)
 		d <- matrix(NA,k,length(x))#rep(NA,k)
 		for(i in 1:k)
@@ -94,8 +90,8 @@ landscapeGeneratorMUL <- function(ref=list(1:10),distanceFunction){
 #' Since the \code{CEGO} package is tailored to minimization, the landscape is inverted.
 #'
 #' @param nGaussian number of Gaussian components in the landscape. Default is 10.
-#' @param upper upper boundary of the \code{distanceFunction}. Controls width of Gaussian components. Default is 1.
-#' @param ratio minimal function value of the local minima. Default is 0.2. (Note: Global minimum will be at zero, local minimal will be in range \code{[ratio;1]})
+#' @param theta controls width of Gaussian components as a multiplier. Default is 1.
+#' @param ratio minimal function value of the local minima. Default is 0.2. (Note: Global minimum will be at zero, local minima will be in range \code{[ratio;1]})
 #' @param seed seed for the random number generator used before creation of the landscape. Generator status will be saved and reset afterwards.
 #' @param distanceFunction A function of type \code{f(x,y)}, to evaluate distance between to samples in their given representation.
 #' @param creationFunction function to randomly generate the centers of the Gaussians, in form of their given representation.
@@ -120,25 +116,23 @@ landscapeGeneratorMUL <- function(ref=list(1:10),distanceFunction){
 #' set.seed(seed)
 #' #landscape
 #' lF <- landscapeGeneratorUNI(cF(),dF)
-#' x <- seq(from=-0,by=0.001,to=1)
+#' x <- as.list(seq(from=0,by=0.001,to=1))
 #' plot(x,lF(x),type="l")
 #' ## multi-modal distance landscape
 #' # set seed
 #' set.seed(seed)
 #' #landscape
 #' lF <- landscapeGeneratorMUL(replicate(5,cF(),FALSE),dF)
-#' x <- seq(from=-0,by=0.001,to=1)
 #' plot(x,lF(x),type="l")
 #' ## glg landscape
 #' #landscape
-#' lF <- landscapeGeneratorGaussian(nGaussian=20,upper=1,
-#' ratio=0.8,seed=seed,dF,cF)
-#' x <- seq(from=-0,by=0.001,to=1)
+#' lF <- landscapeGeneratorGaussian(nGaussian=20,theta=1,
+#' ratio=0.3,seed=seed,dF,cF)
 #' plot(x,lF(x),type="l")
 #'
 #' @export
 ###################################################################################
-landscapeGeneratorGaussian <- function(nGaussian=10,upper=1,ratio=0.2,seed=1, distanceFunction, creationFunction){
+landscapeGeneratorGaussian <- function(nGaussian=10,theta=1,ratio=0.2,seed=1, distanceFunction, creationFunction){
 	## save seed status
 	if(exists(as.character(substitute(.Random.seed))))
 		SAVESEED<-.Random.seed
@@ -147,11 +141,18 @@ landscapeGeneratorGaussian <- function(nGaussian=10,upper=1,ratio=0.2,seed=1, di
 		
 	## set seed
 	set.seed(seed)
-	
+
 	## create landscape
-	fit <- landscapeGeneratorGaussianBuild(nGaussian,upper,ratio,creationFunction)
+	fit <- landscapeGeneratorGaussianBuild(nGaussian,ratio,creationFunction)
 	fit$df <- distanceFunction 
 	
+		
+	# Calculate  maximum distance between centers, for scaling purposes
+	fit$dmax <- max(distanceMatrix(fit$centers,distanceFunction))
+	
+	#save width parameter
+	fit$theta <- theta
+	
 	## load seed status
 	if(!is.null(SAVESEED))
 		assign(".Random.seed", SAVESEED, envir=globalenv())
@@ -159,6 +160,7 @@ landscapeGeneratorGaussian <- function(nGaussian=10,upper=1,ratio=0.2,seed=1, di
 	fit	
 	## create output function	
 	fun <- function(x){
+    if(!is.list(x))x<-list(x)
 		landscapeGeneratorGaussianEval(x,fit)$value
 	}
 	attributes(fun) <- fit
@@ -173,7 +175,6 @@ landscapeGeneratorGaussian <- function(nGaussian=10,upper=1,ratio=0.2,seed=1, di
 #' Core Gaussian landscape function. Should not be called directly, as it does not contain proper seed handling.
 #'
 #' @param nGaussian number of Gaussian components in the landscape. Default is 10.
-#' @param upper upper limit of the distance function to be used, controls the width of Gaussians (variance). Default is 1.
 #' @param ratio minimal function value of the local minima. Default is 0.2. (Note: Global minimum will be at zero, local minimal will be in range \code{[ratio;1]})
 #' @param creationFunction function to randomly generate the centers of the gaussians
 #'
@@ -187,22 +188,16 @@ landscapeGeneratorGaussian <- function(nGaussian=10,upper=1,ratio=0.2,seed=1, di
 #' @export
 #' @seealso \code{\link{landscapeGeneratorGaussian}} 
 ###################################################################################
-landscapeGeneratorGaussianBuild <- function(nGaussian=10,upper=1, ratio=0.2,creationFunction){
+landscapeGeneratorGaussianBuild <- function(nGaussian=10,ratio=0.2,creationFunction){
 	ratio <- 1-ratio
 	if (nGaussian<=0|ratio<=0|ratio>=1){
 		stop('Incorrect parameter values for gaussian landscape generator')
 	}
 
-	varianceRange=upper/20 
-	
-	#variance=matrix(runif(nGaussian*dimension),nGaussian,dimension)*variancerange+0.05;  # add 0.05 to avoid zero variance     #TODO or mat mult?
-	variance <- matrix(runif(nGaussian),nGaussian,1)*varianceRange+0.05;  # add 0.05 to avoid zero variance    
-
-	covmatrix <- variance 
-	covmatrix_inv <- 1/covmatrix
+	variance <- matrix(runif(nGaussian,0.1,0.5),nGaussian,1) # avoid zero variance    
 	
 	# Generate randomly the centers of the gaussians
-	meanVector <- replicate(nGaussian,creationFunction(),simplify=FALSE)
+	centers <- replicate(nGaussian,creationFunction(),simplify=FALSE)
 
 	# assign values to components
 	optimumValue=rep(0,nGaussian) #initialize
@@ -210,7 +205,7 @@ landscapeGeneratorGaussianBuild <- function(nGaussian=10,upper=1, ratio=0.2,crea
 
 	# values of others are randomly generated within [0,ratio]
 	optimumValue[2:nGaussian]=matrix(runif(1*(nGaussian-1)),1,nGaussian-1)*ratio
-	list(centers=meanVector,covinv=covmatrix_inv,opt=1-optimumValue,nGauss=nGaussian)
+	list(centers=centers,covinv= 1/variance,opt=1-optimumValue,nGauss=nGaussian)
 }
 
 ###################################################################################
@@ -230,8 +225,9 @@ landscapeGeneratorGaussianBuild <- function(nGaussian=10,upper=1, ratio=0.2,crea
 #' @seealso \code{\link{landscapeGeneratorGaussian}} 
 ###################################################################################
 landscapeGeneratorGaussianEval <- function(x,glg){
-	covmatrix_inv <- glg$covinv #the inverse covariance matrix of each component
-	meanvector <- glg$centers     #centers of each Gaussian component
+	covinv <- glg$covinv #the inverse covariance matrix of each component
+	theta <- glg$theta #width parameter, multiplier for each gaussian component
+	centers <- glg$centers     #centers of each Gaussian component
 	optimumvalue <- 1-glg$opt   #the peak value of each component
 	nGaussian <-  glg$nGauss  #total number of components
 	
@@ -241,10 +237,10 @@ landscapeGeneratorGaussianEval <- function(x,glg){
 
 	#----------------------------------------------------
 	for(i in 1:nGaussian) {             # calculate the values generated by each component		
-		newx <- distanceVector(meanvector[[i]],x,glg$df)		#x-t(matrix(meanvector[i,],length(meanvector[i,]),p,byrow=FALSE))
-		tmp[i,] <- covmatrix_inv[i]*newx     	   
+		newx <- distanceVector(centers[[i]],x,glg$df)/glg$dmax		#x-t(matrix(centers[i,],length(centers[i,]),p,byrow=FALSE))
+		tmp[i,] <- covinv[i]*newx     	   
 	}
-	f <- exp(-0.5*tmp)            # f is a nGaussian-by-p matrix
+	f <- exp(-theta*tmp)            # f is a nGaussian-by-p matrix
 
 	f <- f*matrix(optimumvalue,length(optimumvalue),p,byrow=FALSE)# multiply the peak value of each component
 	                # the value of each individual generated by each component
